# Class: snmp
#
# This module manages snmp
#
# Parameters: none
#
# Actions:
#
# Requires: see Modulefile
#
# Sample Usage:
#
# Parameters:
# [*log_connection*]
#  Enable or disable connection logs
#  Default: true
# [*ro_community*]
#  Read Only community
#  Default: 'SecureString'
# [*rw_community*]
#  Read Write community
#  Default: 'VerySecureString'
# [*ro_source*]
#  Read Only source
#  Default: '0.0.0.0/0'
# [*rw_source*]
#  Read Write source
#  Default: '127.0.0.1/32'
# [*ro_view*]
#  Read Only view
#  Default: '.1'
# [*rw_view*]
#  Read Write view
#  Default: '.1'
# [*rw_enable*]
#  Enable or Disable Read Write access
#  Default: false
# [*ro_enable*]
#  Enable or Disable Read Only access
#  Default: true
# [*syslocation*]
#  System location
#  Default: 'A
# [*contact_name*]
#  Administrator name
#  Default: 'sysadmin'
# [*contact_mail*]
#  Administrator mail
#  Default: 'sysadmin@example.com'
# [*enable_observium_distro*]
#  Enable or disable observium's distro script
#  Default: false
# [*simple_snmp_enable*]
#  Enable or disable this module, if false, this module will do nothing
#  Default: true

class simple_snmp (
  Boolean $log_connection          = true,
  String $ro_community            = 'SecureString',
  String $rw_community            = 'VerySecureString',
  String $ro_source               = '0.0.0.0/0',
  String $rw_source               = '127.0.0.1/32',
  String $ro_view                 = '.1',
  String $rw_view                 = '.1',
  Boolean $rw_enable               = false,
  Boolean $ro_enable               = true,
  String $syslocation             = 'A beautiful DataCenter',
  String $contact_name            = 'sysadmin',
  String $contact_mail            = 'sysadmin@example.com',
  Boolean $enable_observium_distro = false,
  Boolean $simple_snmp_enable      = true) {
  if $osfamily == 'RedHat' {
    # Check OS release
    if (($::operatingsystemmajrelease == '5') or ($::operatingsystemmajrelease == '6') or ($::operatingsystemmajrelease == '7')) and 
    ($simple_snmp_enable) {
      package { 'simple_snmp_snmpd_package':
        ensure => 'installed',
        name   => 'net-snmp',
      }

      service { 'simple_snmp_snmpd_service':
        ensure => 'running',
        name   => 'snmpd',
        enable => true,
      }

      file { 'simple_snmp_conf':
        path    => '/etc/snmp/snmpd.conf',
        content => template('simple_snmp/snmp_conf.erb'),
        mode    => '0644',
        notify  => Service['simple_snmp_snmpd_service'],
        require => Package['simple_snmp_snmpd_package']
      }

      if ($enable_observium_distro) {
        file { 'simple_snmp_observium_distro_script':
          path   => '/usr/local/bin/distro',
          source => 'puppet:///modules/simple_snmp/observium_distro.sh',
          mode   => '0755',
          notify => Service['simple_snmp_snmpd_service']
        }
      }
    }
  }
}
